/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

//
// Created by hxf on 22-8-2.
//

#ifndef UKUI_SIDEBAR_SIDE_BAR_APPLICATION_H
#define UKUI_SIDEBAR_SIDE_BAR_APPLICATION_H

//注册托盘
#define  TRAY_ICON           ":/icon/kylin-tool-box.svg"
#define  TRAY_NULL_ICON      ":/icon/kylin-tool-box-null.svg"
#define  SETTING_ICON        ":/icon/application-menu.svg"

//注册dbus接口
#define SIDEBAR_CORE_DBUS_SERVICE          "org.ukui.Sidebar"
#define SIDEBAR_CORE_DBUS_PATH             "/org/ukui/Sidebar"
//#define SIDEBAR_CORE_DBUS_INTERFACE        "org.ukui.Sidebar"

#include <QObject>
#include <QUrl>
#include <QQmlApplicationEngine>
#include <QSystemTrayIcon>
#include <QAction>
#include <QMenu>
#include <QProcess>
#include <QTimer>
#include <QDBusInterface>
#include <QObject>
#include <QDBusConnection>
#include <QTranslator>

#include "qtsingleapplication.h"
#include "color-helper.h"
#include "layout-config.h"
#include "plugin-common-data.h"


class SidebarDbusService;
class SidebarEventBridge;
class NotificationEventBridge;
namespace Sidebar {
class SidebarView;
}

namespace UkuiNotification {
    class NotificationCenterWindow;
    class PopupNotificationWindow;
    class TabletPopupView;
}

class SideBarApplication : public QtSingleApplication
{
    friend class SidebarDbusService; //声明d-bus模块为友元类
    friend class SidebarEventBridge;
    friend class NotificationEventBridge;
    Q_OBJECT
public:
    enum Request {
        Active,
        Show,
        Hide,
        DirectlyHide
    };
    explicit SideBarApplication(const QString &appId, int &argc, char **argv);
    ~SideBarApplication();

private Q_SLOTS:
    void startSidebar();
    void execShortcutAction(UkuiShortcut::PluginMetaType::PredefinedAction action);
    void requestSidebar(SideBarApplication::Request request);
    void requestNotificationCenter(SideBarApplication::Request request);

private:
    void parseCommand(const QString &msg, bool isPrimary);
    static void registerItems();

    inline void initPublicObjects();
    void loadQML();
    //系统托盘 设置menu界面、添加动作 和 创建systray实例
    void createSystray();
    void updateSystrayIcon(bool clearRedPoint = true);

    void initSidebar();
    void initNotificationCenter();
    void initPopupWindow();
    void initTabletPopupWindow();

private:
    Sidebar::SidebarView *m_sidebarWindow {nullptr};
    UkuiNotification::NotificationCenterWindow *m_notificationCenterWindow {nullptr};
    UkuiNotification::PopupNotificationWindow *m_popupNotificationWindow {nullptr};
    UkuiNotification::TabletPopupView *m_tabletPopupView {nullptr};

    //系统托盘
    QSystemTrayIcon*            m_trayIcon = nullptr;
    QMenu*                      m_trayIconMenu = nullptr;
    SidebarDbusService*         m_sidebarDbusService = nullptr;
};

class SidebarDbusService : public QObject
{
    Q_OBJECT
    //申明该类有D-BUS服务接口
    Q_CLASSINFO("D-Bus Interface", "org.ukui.Sidebar")
public:
    explicit SidebarDbusService(QObject *parent = nullptr);                    //传入父指针，调用

public Q_SLOTS:
    void sidebarActive();
    void showSidebar(QString arg);
    void hideSidebar(QString arg);

private:
    SideBarApplication* m_sidebarApplication = nullptr;
};

#endif //UKUI_SIDEBAR_SIDE_BAR_APPLICATION_H
