/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Author: hxf <hewenfei@kylinos.cn>
 *
 */

#include "status-bar-view.h"
#include "global-settings.h"
#include "sidebar-window-helper.h"
#include "hand-gesture-helper.h"

StatusBarView::StatusBarView(QWindow *parent) : SharedEngineView(parent)
{
    setColor("transparent");
    setResizeMode(SharedEngineView::SizeRootObjectToView);

    rootContext()->setContextProperty("statusBarView", this);
    rootContext()->setContextProperty("handGestureHelper", Sidebar::HandGestureHelper::getInstance());

    Sidebar::SidebarWindowHelper::setWindowFlags(this, Sidebar::SidebarWindowType::StatusBar);
    Sidebar::SidebarWindowHelper::setWindowAttribute(this, Sidebar::SidebarWindowType::StatusBar);

    updateGeometry();
    connect(Sidebar::SidebarWindowHelper::instance(), &Sidebar::SidebarWindowHelper::geometryChanged,
            this, &StatusBarView::updateGeometry);

    setVisible(isTabletModel());
    connect(Sidebar::GlobalSettings::globalInstance(), &Sidebar::GlobalSettings::valueChanged, this, [this] (const QString &key) {
        if (key == TABLET_MODE) {
            setVisible(isTabletModel());
            Q_EMIT isTabletModelChanged();
        }
    });
}

bool StatusBarView::isTabletModel() const
{
    return Sidebar::GlobalSettings::globalInstance()->getValue(TABLET_MODE).toBool();
}

void StatusBarView::updateGeometry()
{
    Sidebar::SidebarWindowHelper::instance()->updateWindowGeometry(this, Sidebar::SidebarWindowType::StatusBar);
}

bool StatusBarView::event(QEvent *event)
{
    if (event->type() == QEvent::Move) {
        updateGeometry();
        return true;

    } else if (event->type() == QEvent::Expose) {
        Sidebar::SidebarWindowHelper::setWindowAttribute(this, Sidebar::SidebarWindowType::StatusBar);
        updateGeometry();
    }

    return QQuickWindow::event(event);
}

void StatusBarView::init()
{
    setSource(QUrl("qrc:/qml/StatusBar.qml"));
}
